/*
bspfile.h - BSP format similar to Quake3
Copyright (C) 2019 Uncle Mike

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/

#ifndef BSPFILE_NEW_H
#define BSPFILE_NEW_H

/*
==============================================================================

BRUSH MODELS

.wad contain level static geometry with including PVS and lightning info
==============================================================================
*/

#define LUMP_ENTITIES		"entities"
#define LUMP_MATERIALS		"materials"
#define LUMP_PLANES			"planes"
#define LUMP_NODES			"nodes"
#define LUMP_LEAFS			"leafs"
#define LUMP_MARKSURFACES		"marksurfaces"
#define LUMP_MARKBRUSHES		"markbrushes"
#define LUMP_POINTS			"vertexes"
#define LUMP_DRAWVERTS		"drawverts"
#define LUMP_INDEXES		"elements"
#define LUMP_SURFACES		"surfaces"
#define LUMP_MODELS			"submodels"
#define LUMP_BRUSHES		"brushes"
#define LUMP_BRUSHSIDES		"brushsides"
#define LUMP_VISIBILITY		"visdata"

//
// BSP File Structures
//
typedef struct
{
	float		mins[3];
	float		maxs[3];
	float		origin[3];
	int		headnode;
	int		firstface;
	int		numfaces;
	int		firstbrush;
	int		numbrushes;
} dmodel_t;

// planes x^1 is allways the opposite of plane x
typedef struct
{
	float		normal[3];
	float		dist;
} dplane_t;

typedef struct
{
	int		planenum;
	int		children[2];	// negative numbers are -(leafs+1), not nodes
	int		mins[3];		// for frustom culling
	int		maxs[3];
} dnode_t;

typedef struct
{
	int		cluster;		// -1 = opaque cluster (do I still store these?)
	int		area;

	int		mins[3];		// for frustum culling
	int		maxs[3];

	int		firstmarksurface;
	int		nummarksurfaces;

	int		firstmarkbrush;
	int		nummarkbrushes;

	byte		ambients[4];
} dleaf_t;

typedef dword		dbrushside_t;	// positive plane side faces out of the leaf
typedef dword		dmarkelem_t;	// leaf marksurfaces or markbrushes indexes

typedef struct
{
	int		firstside;	// NOTE: first six sides are always axial and brush bevels
	int		numsides;
	int		material;		// material that determine content flags
} dbrush_t;

typedef struct
{
	float		point[3];
} dpoint_t;

// 20 bytes per vert
typedef struct
{
	int		vertex;		// number of indexed vertex
	float		st[2];
	char		normal[4];	// best fit normal + unused entry
	byte		color[4];
} dvertex_t;

typedef enum
{
	TRI_BAD = 0,
	TRI_POLYGON,	// convex points	(also have indexes)
	TRI_LIST,		// indexed list
	TRI_STRIP		// pure strip sequence
} batchtype_t;

typedef struct
{
	int		material;
	batchtype_t	type;

	int		firstvert;
	int		numverts;

	int		firstelem;
	int		numelems;
} dsurface_t;

#endif//BSPFILE_NEW_H